const bodyParser = require('body-parser')

const FragmentsUsers = app => {
  app.use(bodyParser.json())
  const basePath = '/api/collections/:collectionId/fragments/:fragmentId/users'
  const routePath = './routes/fragmentsUsers'
  const authBearer = app.locals.passport.authenticate('bearer', {
    session: false,
  })
  /**
   * @api {post} /api/collections/:collectionId/fragments/:fragmentId/users Add a user to a fragment
   * @apiGroup FragmentsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {fragmentId} fragmentId Fragment id
   * @apiParamExample {json} Body
   *    {
   *      "email": "email@example.com",
   *      "role": "author", [acceptedValues: author],
   *      "isSubmitting": true,
   *      "isCorresponding": false
   *    }
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    {
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email@example.com",
   *      "teams": [
   *        "c576695a-7cda-4e27-8e9c-31f3a0e9d592"
   *      ],
   *      "username": "email@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": false,
   *      "editorInChief": false,
   *      "handlingEditor": false,
   *      "passwordResetToken": "04590a2b7f6c1f37cb84881d529e516fa6fc309c205a07f1341b2bfaa6f2b46c"
   *    }
   * @apiErrorExample {json} Invite user errors
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 500 Internal Server Error
   */
  app.post(
    basePath,
    authBearer,
    require(`${routePath}/post`)(app.locals.models),
  )
  /**
   * @api {delete} /api/collections/:collectionId/fragments/:fragmentId/user/:userId Delete a user from a fragment
   * @apiGroup FragmentsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {fragmentId} fragmentId Fragment id
   * @apiParam {userId} userId User id
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 {}
   * @apiErrorExample {json} Delete errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 404 Not Found
   */
  app.delete(
    `${basePath}/:userId`,
    authBearer,
    require(`${routePath}/delete`)(app.locals.models),
  )
  /**
   * @api {get} /api/collections/:collectionId/fragments/:fragmentId/users List fragment users
   * @apiGroup FragmentsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {fragmentId} fragmentId Fragment id
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    [{
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email@example.com",
   *      "teams": [
   *        "c576695a-7cda-4e27-8e9c-31f3a0e9d592"
   *      ],
   *      "username": "email@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": false,
   *      "editorInChief": false,
   *      "handlingEditor": false,
   *      "passwordResetToken": "04590a2b7f6c1f37cb84881d529e516fa6fc309c205a07f1341b2bfaa6f2b46c",
   *      "isSubmitting": true,
   *      "isCorresponding": false,
   *    },
   *    {
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email2@example.com",
   *      "teams": [
   *        "c576695a-7cda-4e27-8e9c-31f3a0e9d592"
   *      ],
   *      "username": "email2@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": true,
   *      "editorInChief": false,
   *      "handlingEditor": false",
   *      "isSubmitting": false,
   *      "isCorresponding": false,
   *    }]
   * @apiErrorExample {json} List errors
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 404 Not Found
   */
  app.get(basePath, authBearer, require(`${routePath}/get`)(app.locals.models))
  /**
   * @api {patch} /api/collections/:collectionId/fragments/:fragmentId/users/:userId Update a user on a fragment
   * @apiGroup FragmentsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {fragmentId} fragmentId Fragment id
   * @apiParam {userId} userId User id
   * @apiParamExample {json} Body
   *  {
   *    "isSubmitting": false,
   *    "isCorresponding": true,
   *    "firstName": "John",
   *    "lastName": "Smith",
   *    "affiliation": "UCLA"
   *  }
   * @apiSuccessExample {json} Success
   *  HTTP/1.1 200 OK
   *  {
   *    "id": "c95a0f57-780d-4362-9964-7c1af566549b",
   *    "type": "fragment",
   *    "files": {},
   *    "owners": [
   *      {
   *        "id": "4be1d28d-4973-47bc-8732-16d4a1a735ef",
   *        "username": "admin"
   *      }
   *    ],
   *    "created": "2018-05-04T10:50:19.053Z",
   *    "version": 1,
   *    "metadata": {
   *      "title": "<p>Test Manuscript Title</p>",
   *      "abstract": "<p>Test manuscript abstract</p>"
   *    },
   *    "fragmentType": "version",
   *    "authors": [
   *        {
   *          "userId": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *          "isSubmitting": false,
   *          "isCorresponding": true
   *        }
   *      ],
   *   }
   * @apiErrorExample {json} Update invitations errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 404 Not Found
   *    HTTP/1.1 500 Internal Server Error
   */
  app.patch(
    `${basePath}/:userId`,
    authBearer,
    require(`${routePath}/patch`)(app.locals.models),
  )
}

module.exports = FragmentsUsers
